from trainiter import TrainDataIter

RARE_THRESHOLD = 5

class RareList:
    """
    Determines if a given word is rare or not
    Input: train data file name
    """
    
    def __init__(self, fn_train):
        self._word_counts = {}
        for s_tree in TrainDataIter(fn_train):
            self._count_words(s_tree)
        self._non_rares = set()
        for a_key in self._word_counts.keys():
            if self._word_counts[a_key] >= RARE_THRESHOLD:
                self._non_rares.add(a_key)
        self._word_counts = None
    
    def _count_words(self, s_tree):
        list_len = len(s_tree)
        if list_len == 2:
            a_word = s_tree[1]
            if a_word in self._word_counts.keys():
                self._word_counts[a_word] += 1
            else:
                self._word_counts[a_word] = 1
        elif list_len == 3:
            self._count_words(s_tree[1])
            self._count_words(s_tree[2])
        else:
            raise ValueError('sentence tree list have {} items: must be 2 or 3'.format(list_len))
    
    def is_rare(self, a_word):
        return a_word not in self._non_rares

if __name__ == "__main__":
    rare_list = RareList('parse_train.dat')
    print rare_list._non_rares
    test_word = 'colony'
    print 'Is {} rare? - {}'.format(test_word, rare_list.is_rare(test_word))
    test_word = 'Canada'
    print 'Is {} rare? - {}'.format(test_word, rare_list.is_rare(test_word))
   