NONTERMINAL = 'NONTERMINAL'
UNARYRULE = 'UNARYRULE'
BINARYRULE = 'BINARYRULE'
RARE_TAG = '_RARE_'

class RuleParameter:
    """
    returns rule parameter
    """
    def __init__(self, fn_count):
        self._nt_count = {}      # non terminal count
        self._uni_count = {}     # unigram count
        self._em_count = {}      # emission count
        self._non_rare = set()
        word_count = {}
        with open(fn_count, 'r') as fin:
            for l in fin:
                line = l.split()
                count = float(line[0])
                if line[1] == NONTERMINAL:
                   self._nt_count[line[2]] = count
                elif line[1] == UNARYRULE:
                    self._uni_count[line[2], line[3]] = count
                    if line[3] not in word_count.keys():
                        word_count[line[3]] = count
                    else:
                        word_count[line[3]] += count
                elif line[1] == BINARYRULE:
                    self._em_count[line[2], line[3], line[4]] = count
                # else: just ignore invalid data
        for word in word_count.keys():
            if word_count[word] >= 5:
                self._non_rare.add(word)
    def get_nt_list(self):
        return self._nt_count.keys()

    def get_q_uni(self, X, w):
        if w not in self._non_rare:
            w = RARE_TAG
        #if (X, w) not in self._uni_count.keys():
        #    return 0.0
        #else:
        #    return self._uni_count[X, w] / self._nt_count[X]
        try:
            x = self._uni_count[X, w] / self._nt_count[X]
        except KeyError:
            x = 0.0
        return x

    def get_q_em(self, X, Y1, Y2):
        #if (X, Y1, Y2) not in self._em_count.keys():
        #    return 0.0
        #else:
        #    return self._em_count[X, Y1, Y2] / self._nt_count[X]
        try:
            x = self._em_count[X, Y1, Y2] / self._nt_count[X]
        except KeyError:
            x = 0.0
        return x


if __name__ == "__main__":
    rp = RuleParameter('parse_train.counts.out')
    print '_nt_count has ', len(rp._nt_count)
    print '_uni_count has ', len(rp._uni_count)
    print '_em_count has ', len(rp._em_count)
    print rp.get_q_uni('VERB', 'stand')
    print rp.get_q_em('SQ', 'NP', 'VP')
    print rp.get_q_uni('VERB', 'hideki')
    print rp.get_q_em('SQ', 'HIDEKI', 'IKEDA')
    print rp.get_nt_list()
